#!/usr/bin/env ruby

##############################################################
#  Copyright (C) 2007.3 Koji Tsukada
#      All rights reserved.
#      This is free software with ABSOLUTELY NO WARRANTY.
#
#  You can redistribute it and/or modify it under the terms of
#  the GNU General Public License version 2.
##############################################################

require 'getoptlong'

module LedBoard
  class LedBoard
    def initialize
      options = parse_options
      check_options(options)
    
      
      setting = init_setting(options)
      case options['mode']
        when "local"
          require 'localclient'
          LocalClient.new(setting)
        when "web"
          require 'webclient'
          WebClient.new(setting)
        when "mail"
          require 'mailclient'
          MailClient.new(setting)
      end
      loop
    end

    def parse_options
      options = Hash.new
      parser = GetoptLong.new
      parser.set_options(
         ['--mode',	'-M',	GetoptLong::REQUIRED_ARGUMENT],
		     ['--port',       '-p',	GetoptLong::REQUIRED_ARGUMENT],
		     ['--kcode',    	'-k',	GetoptLong::REQUIRED_ARGUMENT],
		     ['--no_scroll',  '-n', GetoptLong::NO_ARGUMENT],
		     ['--no_loop' ,   '-N', GetoptLong::NO_ARGUMENT],
		     ['--vertical',   '-v',	GetoptLong::NO_ARGUMENT],
         ['--reverse',    '-r',	GetoptLong::NO_ARGUMENT],
		     ['--module',     '-m',	GetoptLong::REQUIRED_ARGUMENT],
         ['--bright',     '-b',	GetoptLong::REQUIRED_ARGUMENT],
         ['--help',   '-h',	GetoptLong::NO_ARGUMENT])

      parser.each_option do |name, arg|
        options[name.sub(/^--/, "")] = arg
      end
      return options
    end

    def check_options(options)
      if options['help']
        usage
        exit
      end
      
      if options['mode']
        options['mode'] = options['mode']
      else
        options['mode'] = "local"
      end
    end

    def init_setting(options)
      setting = Hash.new
      #if options['mode']
      #  setting['mode'] = options['mode']
      #else
      #  setting['mode'] = "local"
      #end
    
      if  options['port']
        setting['tty'] = options['port']
      else
        setting['tty'] = "com1"
      end
    
      if options['kcode']
        setting['encoding'] = options['kcode']
      else
        setting['encoding'] = "sjis"
      end
    
      if options['module']
        setting['cmodule'] = options['module']  
      else
        setting['cmodule'] = "2"
      end
    
      if options['bright']
        setting['bright'] = options['bright']
      else
        setting['bright'] = "10"
      end
    
      if options['no_scroll']
        setting['no_scroll'] = 'yes' 
      else
        setting['no_scroll'] = 'no'
      end
  
      if options['no_loop']
        setting['loop'] = 'no'
      else
        setting['loop'] = 'yes'
      end

      if options['vertical']
        setting['vertical'] = 'yes'
        setting['vertical_ank'] = 'yes'
      else
        setting['vertical'] = 'no'
        setting['vertical_ank'] = 'no' 
      end
  
      if options['led_reverse']
        setting['led_reverse'] = 'yes'
      else
        setting['led_reverse'] = 'no'
      end
    
      setting['fonta']  =   '8x16rk.pcf'
      setting['fontk']  =   'jiskan16.pcf'   
    
      setting
    end


    def usage
      puts './LedBoard.rb' 
      puts '  --mode -M [local|mail|web] (default:local)'
      puts '  --port -p [com1-com9] (default:com1)'
	    puts '  --kcode -k [sjis|euc] (default:sjis)'
      puts '  --module -m [1-4] (default:2)'
      puts '  --bright -b [1-10] (default:10)'
	    puts '  --no_scroll -n'
	    puts '  --no_loop -N'
		  puts '  --vertical -v'
      puts '  --reverse -r'  
    end
  
    def loop
      puts "input [q] or [quit] to finish this program" 
  	  while((line = gets)!=nil)
		    break if(line =~ /(quit|q)$/)
		  end
    end
  
  end
  LedBoard.new
end
