##############################################################
#  Copyright (C) 2007.3 Koji Tsukada
#      All rights reserved.
#      This is free software with ABSOLUTELY NO WARRANTY.
#
#  You can redistribute it and/or modify it under the terms of
#  the GNU General Public License version 2.
##############################################################

require 'client'

require 'net/pop'
require 'date'

module LedBoard
  class MailClient  < Client
    MAIL_FILE = "./.mailsetting"
    LOG_FILE = "./.maillog"
  
    #[T[o[ԂƂɊmFD
    def initialize(setting)
      @setting = setting
      mail = load_data(MAIL_FILE)
      mail['key'] = get_key(mail['key'])
      mail['host'] = get_host(mail['host'])
      mail['port'] = get_port(mail['port'])
      mail['encryption'] = get_encryption(mail['encryption'])
      mail['user'] = get_user(mail['user'])
      save_data(MAIL_FILE, mail)
      mail['pass'] = get_pass
      system("clear")
      
      mail_loop(mail)
    end
    
    def get_host(default)
      print("mail server(default=#{default}): ")
	    host = gets.chomp
      if host == nil || host == ""
        host = default
      end
      host
    end
    
    def get_user(default)
      print("user(default=#{default}): ")
      user = gets.chomp 
      if user == nil || user == ""
        user = default
      end
      user
    end
    
    def get_port(default)
      print("port(default=#{default}): ")
      port = gets.chomp 
      if port == nil || port == ""
        port = default
      end
      port
    end
    
    def get_pass
      print("password: ")
      pass = gets.chomp 
    end
    
    def get_key(default)
      default = "[LED]" if default == "" || default == nil
      print("keyword in subject(default=#{default}): ")
      key = gets.chomp
      if key == nil || key == ""
        key = default
      end
      key
    end
    
    def get_encryption(default)
      result = true
      print("encryption enabled [true|false](default=#{default}): ")
      if gets.chomp =~ /false/i
        result = false
      end
      result  
    end
    
    def mail_loop(mail, sleep_time = 10)
      begin
        Thread.start do
          while true
            pop(mail)
            sleep(sleep_time)
          end
        end
      rescue
        puts("error: " + $!)
        exit(0)
      end
    end
    
    def pop(mail)
     
      Net::POP3.APOP(mail['encryption']).new(mail['host'], mail['port'].to_i).start(mail['user'], mail['pass']) do |pop|
        puts "mail cnt:" + pop.mails.length.to_s
        if pop.mails.empty? then
          #puts 'no mail.'
        else
          pop.mails.each do |m|
            if match_key?(m.header, mail['key'])
              puts "found key"
              header, body = m.pop.gsub(/\r/,"").split(/\n\n/)
              puts header 
              puts "---"
              puts body
              message, speed, other = body.split(/\n/)
              message = nkf(message)
              speed = "" if speed == nil      
              write(message.chomp, speed.chomp)
              save_log(header, message, speed)
              m.delete
              break
            end 

          end
        end 
      end
    end
    
    def save_log(header, message, speed)

       d = DateTime.parse( find_value(header, "Date"))
       d_str = "#{d.year}/#{d.month}/#{d.day} #{d.hour}:#{d.min}:#{d.sec}"
       puts d_str
       from = find_value(header, "From")
       puts from
       File.open(LOG_FILE, "a") do |file|
          file.puts "#{d_str}\t#{from}\t#{message}\t#{speed}"
       end 
    end
          
    def match_key?(header, key)
      result = false
      value = find_value(header, "Subject")
      if value =~ /#{key}/
        #puts "found: " + args[1]
        result = true
      end
      result
    end
    
    def find_value(header, name)
      result = nil
      headers = header.gsub(/\r/,"").split("\n")
      headers.each do |h|
        next if h == nil ||  h == ""
        args = h.chomp.split(": ")
        next if args == nil || args.length < 2
        if args[0] =~ /^#{name}/i
          result = args[1]
        end
      end
      result
    end
    
  end
end
