##############################################################
#  Copyright (C) 2007.3 Koji Tsukada
#      All rights reserved.
#      This is free software with ABSOLUTELY NO WARRANTY.
#
#  You can redistribute it and/or modify it under the terms of
#  the GNU General Public License version 2.
##############################################################

require 'client'

require 'net/http'
require 'uri'
require 'rss'

module LedBoard
  class WebClient  < Client
    WEB_FILE = "./.websetting"
    
    def initialize (setting)
      @setting = setting
      web = load_data(WEB_FILE)
      if(web != nil && web['url'] != nil)
        default_url = web['url']
      else
        default_url = "none"
      end
      uri = get_uri(default_url)
      speed = get_speed
      save_data(WEB_FILE, {"url"=>uri.to_s})
      modified_time = Time.at(0)
      web_loop(uri, speed, modified_time)
    end
    
   
    def get_uri(default_url)
       print("target url(default:#{default_url})")
       url = gets.chomp
       url = default_url if url == nil || url == ""
       uri = URI.parse(url)
    end
    
    def get_speed
      print("speed[1-10]: ")
      speed = gets.chomp.to_i
    end
    
    def web_loop(uri, speed, modified_time, sleep_time = 10)
      Thread.start do
        begin
        while true
          time  = get_modified_time(uri)
          #puts uri.to_s + " " + time.to_s
          if(time != nil && modified_time < time)
            puts "updated: " + uri.to_s + " " + time.to_s
            #message, speed = open_uri(uri)
            message = open_uri(uri)
            write(message, speed)
            modified_time = time
          end
          sleep(sleep_time)
        end
        rescue
          puts("Warning: #$!")
        end
      end
    end

    def open_uri(uri, timeout = 2)
      message = nil
      speed = 3
      if(uri.path =~ /[.]rdf$/)
        message = parse_rss(uri)
      else
        message, speed = parse_uri(uri)  
      end
      message
      #[message, speed]
    end
  
    def parse_uri(uri)
      http = http_connect(uri)
      response = http.get(uri.path)
      body = nkf(response.body)
      #puts body
      args = body.gsub("\r", "").split("\n")
      #puts args[0] + " " + args[1]
      [args[0], args[1]]
    end
       
    def parse_rss(uri)
      http = http_connect(uri)
      response = http.get(uri.path)
      rss = RSS::Parser.parse(response.body)
      message = nkf(rss.items[0].description)
      puts message
      message
    end
    
    def get_modified_time (uri, timeout = 2)
      return nil unless /^http/ =~ uri.to_s
      time = nil
      dead = Time.at(0)
      begin
        http = http_connect(uri, timeout)
        response = http.head(uri.path)
        if response.code.to_i != 200
          time = dead
        elsif response['last-modified']
          time = Time.httpdate(response['last-modified']) 
        else
          time = Time.now 
        end
      rescue Net::ProtoFatalError => e  # 404
        puts "error:not found"
        time = dead
      rescue
        puts "error:timeout"
        time = dead # timeout? 
      end
      time
    end
    
    def http_connect(uri, timeout = 2)
      http =  Net::HTTP.new(uri.host, uri.port)
      http.open_timeout = timeout
      http.read_timeout = timeout
      http
    end
  end
end
