#ifdef  __CYGWIN__
#include <windows.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "sio.h"
#include "nabort.h"

static int  si;
static char sio_buf[1024];

void sio_init(sio_t *scont)
{
#ifndef  __CYGWIN__
  scont->tty_name    = "/dev/ttyS0";
  scont->tty_h = -1;
#else
  scont->tty_name    = "COM1";
  scont->comHandle   = INVALID_HANDLE_VALUE;
#endif  
  scont->tty_baud    = 19200;
  scont->f_v  = 0;
}

void sio_input_purge(sio_t *scont)
{
  char buf[128];
  int  wc,n;

  for(wc=0;wc<10;wc++){
    n = sio_read(scont, buf, sizeof(buf));
    if(n < 0) nabort("sio_input_purge:Error In sio_read");
    if(n > 0) wc = 0;
    usleep(10000);
  }
}

/**
 *  1ǂݍ(҂Ȃ)
 */
int sio_getc0(sio_t *scont,unsigned char *pC)
{
  int n;
  char buf[10];

  n = sio_read(scont, buf, sizeof(buf));
  if(n < 0) nabort("sio_getc0: error in sio_read\n");

  if(n > 0) {
    *pC = buf[0];
    return 1;
  }
  else return 0;
}

/**
 *  1ǂݍ,0.2b͂Ȃ΃^CAEg
 *
 */
int sio_getc(sio_t *scont)
{
  unsigned char c;
  int cnt,t;

  for(t=0;t<200;t++){
    for(cnt=0;cnt<100;cnt++){
      if(sio_getc0(scont, &c))
	return c;
    }
    usleep(1000);
  }
  nabort("sio_getc: timeout\n");
  return 0;
}

void sio_putc(sio_t *scont, int c)
{
  int n;
  char buf[10];
  
  buf[0] = c;

  n = sio_write(scont, buf, 1); 
  if(n < 0) {
    perror("sio_putc");
    nabort("io error");
  }
}

void sio_putc_e(sio_t *scont,int c)
{
  int e;
  c &= 0xff;
  sio_putc(scont,c);
  e = sio_getc(scont);
  if(c != e){
    nabort("sio_putc_e: transmit %02x, recieve %02x\n", c, e);
  }
}

void sio_puts(sio_t *scont, const char *str)
{
  int len = strlen(str);
  sio_write(scont, str, len);
}

char *sio_gets(sio_t *scont, char *buf, int buflen)
{
  char *p;
  int len,n,wc;

  if(si > 0){
    sio_buf[si] = 0;
    p = strchr(sio_buf,'\n');
    if(p != NULL){
      len = p - sio_buf + 1;
      if(len > buflen) len = buflen;
      memcpy(buf, sio_buf, len);
      buf[len] = 0;
      memcpy(sio_buf, sio_buf + len, si - len);
      si -= len;
      return buf;
    }
  }
  if(si >= buflen-1){
    len = buflen-1;
    memcpy(buf, sio_buf, len);
    buf[len] = 0;
    memcpy(sio_buf, sio_buf+len, len);
    si -= buflen;
    return buf;
  }

  for(wc=0; wc < 100; wc++){
    len = sizeof(sio_buf) - si - 1;
    n = sio_read(scont, sio_buf+si, len);
    if(n > 0){
      wc = 0;
      sio_buf[si+n] = 0;
      p = strchr(sio_buf + si,'\n');
      si += n;
      
      if(p != NULL){
	len = p - sio_buf + 1;
	if(len >= buflen - 1) len = buflen-1;
	memcpy(buf, sio_buf, len);
	buf[len] = 0;
	if(si + n - len > 0)
	  memcpy(sio_buf, sio_buf+len, si + n - len);
	si -= len;
	return buf;
      }
      if(si >= buflen-1){
	len = buflen - 1;
	memcpy(buf, sio_buf, len);
	memcpy(sio_buf, sio_buf+len, buflen);
	si -= len;
	return buf;
      }
    }
    else if(n == 0) 
      usleep(10000);
    else {
      perror("sio_gets");
      nabort("io error\n");
    }
  }
  return NULL;
}

void sio_sync(sio_t *scont)
{
  char buf[128];

  sio_puts(scont, "\r");
  usleep(10000);
  
  sio_puts(scont,"echo 0\r");
  sio_input_purge(scont);
  sio_puts(scont,"sync\r");

  while(1){
    if(sio_gets(scont, buf, sizeof(buf))==NULL)
      nabort("connection with H8CON failed.");
    if(strncmp("D SYNC",buf,5)==0)
      break;
  }
  while(1){
    if(sio_gets(scont, buf, sizeof(buf))==NULL)
      nabort("H8CON bad response.");
    if(strncmp("Ok",buf,2)==0)
      break;
  }
}

char *sio_cmd(sio_t *scont,const char *cmd_fmt, ...)
{
  va_list ap;
  char cmd[128];
  char buf[128];
  static char res[128];

  va_start(ap, cmd_fmt);
  vsnprintf(cmd, sizeof(buf), cmd_fmt, ap);
  va_end(ap);

  if(scont->f_v > 1) fprintf(stderr,"cmd:%s\n",cmd);
  sio_puts(scont,cmd);
  sio_puts(scont,"\r");
  res[0] = 0;
  
  while(1){
    if(sio_gets(scont, buf, sizeof(buf))==NULL)
      nabort("response timeout to \"%s\"\n",cmd);
    if(strncmp("D ",buf,2)==0 && res != NULL){
      strcpy(res, buf);
    }
    else if(buf[0]=='E' || buf[0] == 'V'){
      if(scont->f_v > 1)
	fprintf(stderr,"H8 mes:%s",buf);
    }
    else if(strncmp("Ok",buf,2)==0)
      break;
  }
  return res;
}

